require_dependency "asset_management/asset_management_controller"

module AssetManagement
  class AssetDataController < AssetManagementController
    before_action :set_asset_data, only: [:show, :edit, :update, :destroy]

    # GET /asset_data
    # GET /asset_data.json
    def index
      @asset_data = AssetData.all

      respond_to do |format|
        format.html # index.html.erb
        format.json { render :json => @asset_data }
      end
    end

    # GET /asset_data/1
    # GET /asset_data/1.json
    def show
      respond_to do |format|
        format.html # show.html.erb
        format.json { render :json => @asset_data }
      end
    end

    # GET /asset_data/new
    # GET /asset_data/new.json
    def new
      @asset_data = AssetData.new

      respond_to do |format|
        format.html # new.html.erb
        format.json { render :json => @asset_data }
      end
    end

    # GET /asset_data/1/edit
    def edit
    end

    # POST /asset_data
    # POST /asset_data.json
    def create
      params[:asset_data][:filename] = params[:asset_data][:asset_file].original_filename.chomp if(params[:asset_data][:asset_file].class == 'ActionDispatch::Http::UploadedFile')
      @asset_data = AssetData.new(asset_data_params)
      #@asset_data.owner_id = 0
      #@asset_data.owner = current_user if defined(current_user)

      respond_to do |format|
        if @asset_data.save
          format.html { redirect_to @asset_data, :notice => 'Asset data was successfully created.' }
          format.json { render :json => @asset_data, :status => :created, :location => @asset_data }
        else
          format.html { render :action => "new" }
          format.json { render :json => @asset_data.errors, :status => :unprocessable_entity }
        end
      end
    end

    # PUT /asset_data/1
    # PUT /asset_data/1.json
    def update
      #@asset_data.owner_id = 0
      #@asset.owner = current_user unless current_user.nil?

      respond_to do |format|
        if @asset_data.update_attributes(asset_data_params)
          format.html { redirect_to @asset_data, :notice => 'Asset data was successfully updated.' }
          format.json { head :no_content }
        else
          format.html { render :action => "edit" }
          format.json { render :json => @asset_data.errors, :status => :unprocessable_entity }
        end
      end
    end

    # DELETE /asset_data/1
    # DELETE /asset_data/1.json
    def destroy
      @asset_data.destroy

      respond_to do |format|
        format.html { redirect_to asset_data_index_url }
        format.json { head :no_content }
      end
    end

    private
      # Use callbacks to share common setup or constraints between actions.
      def set_asset_data
        @asset_data = AssetData.find(params[:id])
      end

      # Never trust parameters from the scary internet, only allow the white list through.
      def asset_data_params
        params.require(:asset_data).permit(
          :name,
          :filename,
          :data,
          :mime_type,
          :size,
          :description,
          :folder,
          :folder_id,
          :storage_reference,
          :asset_file
        )
      end
  end
end
