require_dependency "asset_management/asset_management_controller"

module AssetManagement
  class ViewController < AssetManagementController

    def original
      # display original in all it's glory
      unless params[:filepath].blank?

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        if asset_data.mime_type =~ /^image/
          begin
            image_data = load_magick_image(asset_data)

            send_data image_data.to_blob(),
              :filename => asset_data.filename, :type => asset_data.mime_type,
              :size => asset_data.size, :disposition => "inline"
          rescue MiniMagick::Error
            logger.error "ERROR: #{$!}\n"
            logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
          # TODO: remove - RMagick
          # rescue Magick::ImageMagickError
          #   logger.error "ERROR: " + $! + "\n"
          #   logger.error "#{$0}: ImageMagickError - #{$!}"
          end
        else
          send_data asset_data.data,
            :filename => asset_data.filename, :type => asset_data.mime_type,
            :size => asset_data.size, :disposition => "inline"
        end
      else
        render :text => 'File not found!'
      end
    end

    def thumbnail
      # asset_data.thumbnail(width, height)
      unless params[:filepath].blank?

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        unless params[:size].nil?
          width  = params[:size].split("x").first.to_i
          height = params[:size].split("x").last.to_i
        else
          width  = 75
          height = 75
        end

        begin
          original_image = load_magick_image(asset_data)
          image_data = original_image.thumbnail "#{width}x#{height}^"

          # TODO: remove - RMagick
          # image_data = original_image.thumbnail(width, height)

          send_data image_data.to_blob(),
            :filename => asset_data.filename, :type => asset_data.mime_type,
            :size => asset_data.size, :disposition => "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
        # TODO: remove - RMagick
        # rescue Magick::ImageMagickError
        #   logger.error "ERROR: " + $! + "\n"
        #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      else
        render :text => 'File not found!'
      end
    end

    def flickr_thumbnail
      # asset_data.resize_to_fill(width, height)
      unless params[:filepath].blank?

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        unless params[:size].nil?
          width  = params[:size].split("x").first.to_i
          height = params[:size].split("x").last.to_i
        else
          width  = 75
          height = 75
        end

        begin
          original_image = load_magick_image(asset_data)
          image_data = original_image.resize "#{width}x#{height}!"

          # TODO: remove - RMagick
          # image_data = original_image.resize_to_fill(width, height)

          send_data image_data.to_blob(),
            :filename => asset_data.filename, :type => asset_data.mime_type,
            :size => asset_data.size, :disposition => "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
        # TODO: remove - RMagick
        # rescue Magick::ImageMagickError
        #   logger.error "ERROR: " + $! + "\n"
        #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      else
        render :text => 'File not found!'
      end
    end

    def scaled
      # asset_data.resize_to_fit(width, height)
      unless params[:filepath].blank?

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        begin
          original_image = load_magick_image(asset_data)

          unless params[:size].nil?
            width  = params[:size].split("x").first.to_i
            height = params[:size].split("x").last.to_i
            image_data = original_image.resize "#{width}x#{height}"

            # TODO: remove - RMagick
            # image_data = original_image.resize_to_fit(width, height)
          else
            image_data = original_image
          end

          send_data image_data.to_blob(),
            :filename => asset_data.filename, :type => asset_data.mime_type,
            :size => asset_data.size, :disposition => "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
        # TODO: remove - RMagick
        # rescue Magick::ImageMagickError
        #   logger.error "ERROR: " + $! + "\n"
        #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      else
        render :text => 'File not found!'
      end
    end

    def scale
      # asset_data.resize(scale_factor)
      unless params[:filepath].blank?

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        begin
          original_image = load_magick_image(asset_data)

          unless params[:magnification].nil?
            scale_factor  = params[:magnification].to_f / 100.0
            image_data = original_image.resize "#{scale_factor*100}%"

            # TODO: remove - RMagick
            # image_data = original_image.resize(scale_factor)
          else
            image_data = original_image
          end

          send_data image_data.to_blob(),
            :filename => asset_data.filename, :type => asset_data.mime_type,
            :size => asset_data.size, :disposition => "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
        # TODO: remove - RMagick
        # rescue Magick::ImageMagickError
        #   logger.error "ERROR: " + $! + "\n"
        #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      else
        render :text => 'File not found!'
      end
    end

    private

    def find_asset_data_from_file_path(full_path)
      #directory = File.dirname(full_path)
      filename  = File.basename(full_path)

      AssetData.find_by(:filename => filename)
    end

    def load_magick_image(asset_data)
      (asset_data.nil?) ? nil : asset_data.load_magick_image
    end

  end
end
