require_dependency "asset_management/asset_management_controller"

module AssetManagement
  class VirtualFoldersController < AssetManagementController
    before_action :set_virtual_folder, only: [:show, :edit, :update, :destroy]

    # GET /virtual_folders
    # GET /virtual_folders.json
    def index
      @virtual_folders = VirtualFolder.all

      respond_to do |format|
        format.html # index.html.erb
        format.json { render :json => @virtual_folders }
      end
    end

    # GET /virtual_folders/1
    # GET /virtual_folders/1.json
    def show
      respond_to do |format|
        format.html # show.html.erb
        format.json { render :json => @virtual_folder }
      end
    end

    # GET /virtual_folders/new
    # GET /virtual_folders/new.json
    def new
      @virtual_folder = VirtualFolder.new

      respond_to do |format|
        format.html # new.html.erb
        format.json { render :json => @virtual_folder }
      end
    end

    # GET /virtual_folders/1/edit
    def edit
    end

    # POST /virtual_folders
    # POST /virtual_folders.json
    def create
      @virtual_folder = VirtualFolder.new(virtual_folder_params)

      respond_to do |format|
        if @virtual_folder.save
          format.html { redirect_to @virtual_folder, :notice => 'Virtual folder was successfully created.' }
          format.json { render :json => @virtual_folder, :status => :created, :location => @virtual_folder }
        else
          format.html { render :action => "new" }
          format.json { render :json => @virtual_folder.errors, :status => :unprocessable_entity }
        end
      end
    end

    # PUT /virtual_folders/1
    # PUT /virtual_folders/1.json
    def update
      respond_to do |format|
        if @virtual_folder.update_attributes(virtual_folder_params)
          format.html { redirect_to @virtual_folder, :notice => 'Virtual folder was successfully updated.' }
          format.json { head :no_content }
        else
          format.html { render :action => "edit" }
          format.json { render :json => @virtual_folder.errors, :status => :unprocessable_entity }
        end
      end
    end

    # DELETE /virtual_folders/1
    # DELETE /virtual_folders/1.json
    def destroy
      @virtual_folder.destroy

      respond_to do |format|
        format.html { redirect_to virtual_folders_url }
        format.json { head :no_content }
      end
    end

    private
      # Use callbacks to share common setup or constraints between actions.
      def set_virtual_folder
        @virtual_folder = VirtualFolder.find(params[:id])
      end

      # Never trust parameters from the scary internet, only allow the white list through.
      def virtual_folder_params
        params.require(:virtual_folder).permit(
          :name,
          :description,
          :parent_folder,
          :parent_folder_id
        )
      end
  end
end
