module AssetManagement
  class VirtualFolder < ActiveRecord::Base

    ROOT = 'ROOT'

    #---------------------------------------------------------------------------

    self.table_name = 'asset_management_virtual_folders'

    belongs_to :parent_folder, :class_name => '::AssetManagement::VirtualFolder', :dependent => :destroy
    has_many :folders, :class_name => '::AssetManagement::VirtualFolder', :foreign_key => :parent_folder_id

    has_many :assets, :class_name => '::AssetManagement::AssetData', :foreign_key => :folder_id

    #----------------------------------------------------------------------------

    # attr_accessible :name
    # attr_accessible :description
    # attr_accessible :parent_folder, :parent_folder_id

    #----------------------------------------------------------------------------

    validates :name, :presence => true
    #validates :parent_folder, :presence => true

    #----------------------------------------------------------------------------

    def to_s
      name
    end

    def subfolders
      ::AssetManagement::VirtualFolder.where(:parent_folder_id => self.id).where("id != #{self.id}")

      #result = []
      #AssetManagement::VirtualFolder.where(:parent_folder_id => self.id).each do |virtual_folder|
      #  result << virtual_folder if virtual_folder.id != self.id
      #end
      #result
    end

    def assets
      ::AssetManagement::AssetData.where(:folder_id => self.id)
    end

    def self.jstree_structure(folder = nil)
      result = []
      children = []

      root_folder = ::AssetManagement::VirtualFolder.find_by(:name => ::AssetManagement::VirtualFolder::ROOT)
      folder = root_folder if folder.nil?

      # Iterate sub-folders
      folder.subfolders.each do |subfolder|
        children << jstree_structure(subfolder) unless folder.subfolders.empty?
      end

      # Generate 'id' for this node
      path = folder.name

      current_folder = folder
      while current_folder.parent_folder != root_folder
        current_folder = current_folder.parent_folder
        path = "#{current_folder.name}:::#{path}"
      end

      if folder.assets.length > 0
        folder.assets.each do |asset|

          file_attr = {
            :id => "#{path}:::#{asset.filename}",
            :rel => 'file',
            :href => ::AssetManagement::Engine.routes.url_for({
                :controller => 'asset_management/view', :action => 'original',
                :filepath => asset.filename, :only_path => true
              })
          }

          children << {
            :data => asset.filename,
            :attr => file_attr
          }
        end
      end

      folder_attr = { :id => "#{path}:::#{folder.id}", :rel => 'folder' }

      if children.length > 0
        result << { :data => folder.name, :attr => folder_attr, :children => children }
      else
        result << { :data => folder.name, :attr => folder_attr }
      end

      result
    end

  end
end
