# To use the mattr functions, ActiveSupport dependencies are required.
require 'rails'
require 'active_support/dependencies'

module AssetManagement

  # The parent controller all engine controllers inherits from.
  # Defaults to ApplicationController. This should be set early
  # in the initialisation process and should be set to a string.
  mattr_accessor :parent_controller
  self.parent_controller = "ApplicationController"

  # Our host application root path
  # We set this when the engine is initialized
  # mattr_accessor :app_root
  # self.app_root = 'xyz'

  # Allow clients to customise the storage backend.
  #
  # NOTE: If not set, the default is to use the :database storage backend.
  #
  # :type
  # The storage backend to use
  # Valid values: :database, :disk, :cloud
  #
  # :disk_directory
  # The directory on the disk to use
  #
  # :cloud_provider
  # The cloud storage provider
  # Valid values: :aws, :owncloud
  #
  # :cloud_provider_auth_user
  #
  # :cloud_provider_auth_password
  #
  # Clients may register column settings like this:
  #   storage = { :type => :disk,
  #               :disk_directory => Dir(Rails.root,'app','assets','data')
  #             }
  #
  #   storage = { :type => :cloud,
  #               :cloud_provider => :aws,
  #               :cloud_provider_auth_user => 'userId',
  #               :cloud_provider_auth_password = 'password'
  #             }
  #
  mattr_accessor :storage
  self.storage ||= {}

  # Default way to setup the engine.
  #
  # Run 'rails generate accounts_management_install' to create a fresh initializer with all
  # configuration values.
  def self.setup
    yield self
  end

  def railtie_name
    AssetManagement::Engine.railtie_name
  end

  def mounted_path
    AssetManagement::Engine.mounted_path
  end

  def root_path
    AssetManagement::Engine.root
  end
end

# Require our engine
require "asset_management/version"
require "asset_management/engine"
